import json
import grpc
import jwt
import datetime
from decouple import config
from .stubs import hive_pb2, hive_pb2_grpc
from elastos_adenine.settings import REQUEST_TIMEOUT, TOKEN_EXPIRATION
import base64
from cryptography.fernet import Fernet
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC


class Hive:

    def __init__(self, host, port, production):
        if not production:
            self._channel = grpc.insecure_channel('{}:{}'.format(host, port))
        else:
            credentials = grpc.ssl_channel_credentials()
            self._channel = grpc.secure_channel('{}:{}'.format(host, port), credentials)

        self.stub = hive_pb2_grpc.HiveStub(self._channel)

    def close(self):
        self._channel.close()

    def upload_and_sign(self, api_key, did, network, private_key, filename):
        with open(filename, 'rb') as myfile:
            file_contents = myfile.read()

        # encoding and encrypting
        key = get_encrypt_key(private_key)
        fernet = Fernet(key)
        encrypted_message = fernet.encrypt(file_contents)

        # generate JWT token
        jwt_info = {
            'network': network,
            'privateKey': private_key,
            'file_content': encrypted_message.decode("utf-8") 
        }

        jwt_token = jwt.encode({
            'jwt_info': jwt_info,
            'exp': datetime.datetime.utcnow() + datetime.timedelta(hours=TOKEN_EXPIRATION)
        }, api_key, algorithm='HS256')


        response = self.stub.UploadAndSign(hive_pb2.Request(input=jwt_token), timeout=REQUEST_TIMEOUT, metadata=[('did', did)])
        
        if response.status:
            output = jwt.decode(response.output, key=api_key, algorithms=['HS256']).get('jwt_info')
            result = {
                'output': json.dumps(output),
                'status_message': response.status_message,
                'status': response.status
            }
            return result
        else:
            result = {
                'output': '',
                'status_message': response.status_message,
                'status': response.status
            }
            return result

    def verify_and_show(self, api_key, did, network, request_input):
        jwt_info = {
            'network': network,
            'request_input': request_input
        }

        jwt_token = jwt.encode({
            'jwt_info': jwt_info,
            'exp': datetime.datetime.utcnow() + datetime.timedelta(hours=TOKEN_EXPIRATION)
        }, api_key, algorithm='HS256')

        response = self.stub.VerifyAndShow(hive_pb2.Request(input=jwt_token), timeout=REQUEST_TIMEOUT, metadata=[('did', did)])

        if response.status:
            jwt_info = jwt.decode(response.output, key=api_key, algorithms=['HS256']).get('jwt_info')
            file_content = jwt_info['file_content']

            #decrypt file content
            key = get_encrypt_key(request_input['privateKey'])
            fernet = Fernet(key)
            decrypted_message = fernet.decrypt(file_content.encode()) 
            
            result = {
                'file_content': decrypted_message,
                'status_message': response.status_message,
                'status': response.status
            }
            return result
        else:
            result = {
                'output': '',
                'status_message': response.status_message,
                'status': response.status
            }
            return result

def get_encrypt_key(key):
    encoded = key.encode()
    salt = config('ENCRYPTION_SALT').encode()
    kdf = PBKDF2HMAC(
        algorithm=hashes.SHA256(),
        length=32,
        salt=salt,
        iterations=100000,
        backend=default_backend()
    )
    key = base64.urlsafe_b64encode(kdf.derive(encoded))
    return key



import grpc
import jwt
import datetime
from decouple import config
from .stubs import node_rpc_pb2, node_rpc_pb2_grpc
from elastos_adenine.settings import REQUEST_TIMEOUT, TOKEN_EXPIRATION


class NodeRpc:

    def __init__(self, host, port, production):
        if not production:
            self._channel = grpc.insecure_channel('{}:{}'.format(host, port))
        else:
            credentials = grpc.ssl_channel_credentials()
            self._channel = grpc.secure_channel('{}:{}'.format(host, port), credentials)

        self.stub = node_rpc_pb2_grpc.NodeRpcStub(self._channel)

    def close(self):
        self._channel.close()

    # TODO: Common method for mainchain only
    def get_current_current_council(self, network):
        params = {"state": "all"}
        return self.rpc_method(network, "mainchain", "listcurrentcrs", params)

    # TODO: Common method for mainchain only
    def get_current_crc_candidates(self, network):
        params = {"start": 0, "state": "all"}
        return self.rpc_method(network, "mainchain", "listcrcandidates", params)

    # TODO: Common method for mainchain only
    def get_current_dpos_supernodes(self, network):
        params = {"start": 0, "state": "all"}
        return self.rpc_method(network, "mainchain", "listproducers", params)

    # Common method for mainchain only
    def get_current_arbitrator_group(self, network):
        params = {"height": str(self.get_current_height(network, "mainchain"))}
        return self.rpc_method(network, "mainchain", "getarbitratorgroupbyheight", params)

    # Common method for mainchain only
    def get_arbitrator_group(self, network, height):
        params = {"height": height}
        return self.rpc_method(network, "mainchain", "getarbitratorgroupbyheight", params)

    # Common method for mainchain only
    def get_current_arbitrators_info(self, network):
        return self.rpc_method(network, "mainchain", "getarbitersinfo", {})

    # Common method for mainchain only
    def get_current_block_confirm(self, network):
        params = {"height": self.get_current_height(network, "mainchain"), "verbosity": 1}
        return self.rpc_method(network, "mainchain", "getconfirmbyheight", params)

    # Common method for mainchain only
    def get_block_confirm(self, network, height):
        params = {"height": height, "verbosity": 1}
        return self.rpc_method(network, "mainchain", "getconfirmbyheight", params)

    # Common method for mainchain only
    def get_current_mining_info(self, network):
        return self.rpc_method(network, "mainchain", "getmininginfo", {})

    # Common method for mainchain, did sidechain and token sidechain
    def get_current_block_details(self, network, chain):
        params = {"height": str(self.get_current_height(network, chain))}
        return self.rpc_method(network, chain, "getblockbyheight", params)

    # Common method for mainchain, did sidechain and token sidechain
    def get_block_details(self, network, chain, height):
        params = {"height": height}
        return self.rpc_method(network, chain, "getblockbyheight", params)

    # Common method for mainchain, did sidechain and token sidechain
    def get_current_balance(self, network, chain, address):
        params = {"address": address}
        return self.rpc_method(network, chain, "getreceivedbyaddress", params)

    # Common method for mainchain, did sidechain and token sidechain
    def get_current_height(self, network, chain):
        node_state = self.get_current_node_state(network, chain)
        current_height = node_state["height"]
        return current_height

    # Common method for mainchain, did sidechain and token sidechain
    def get_current_node_state(self, network, chain):
        return self.rpc_method(network, chain, "getnodestate", {})

    def rpc_method(self, network, chain, method, params):
        secret_key = config('SHARED_SECRET_ADENINE')
        req_data = {
            'chain': chain,
            'method': method,
            'params': params
        }

        jwt_info = {
            'network': network,
            'request_input': req_data
        }

        jwt_token = jwt.encode({
            'jwt_info': jwt_info,
            'exp': datetime.datetime.utcnow() + datetime.timedelta(hours=TOKEN_EXPIRATION)
        }, secret_key, algorithm='HS256')

        response = self.stub.RpcMethod(node_rpc_pb2.Request(input=jwt_token), timeout=REQUEST_TIMEOUT)
        data = {}

        if response.status:
            output = jwt.decode(response.output, key=secret_key, algorithms=['HS256']).get('jwt_info')
            data = output['result']
        
        return data




import pytest
import sys
from decouple import config
import json
import argparse

from elastos_adenine.stubs import health_check_pb2
from elastos_adenine.health_check import HealthCheck
from elastos_adenine.sidechain_eth import SidechainEth
from elastos_adenine.common import Common

host = config('GRPC_SERVER_HOST')
port = config('GRPC_SERVER_PORT')
production = config('PRODUCTION', default=False, cast=bool)

network = "gmunet"
mnemonic_to_use = 'obtain pill nest sample caution stone candy habit silk husband give net'
did_to_use = 'n84dqvIK9O0LIPXi27uL0aRnoR45Exdxl218eQyPDD4lW8RPov'
api_key_to_use = ''
private_key_to_use = '1F54BCD5592709B695E85F83EBDA515971723AFF56B32E175F14A158D5AC0D99'


def test_sidechain():
    # test_sidechain
    # assert response.status==True,"Testing test_sidechain Failed"
    pass



import pytest
import sys
from decouple import config
import json
import argparse

from elastos_adenine.stubs import health_check_pb2
from elastos_adenine.health_check import HealthCheck
from elastos_adenine.wallet import Wallet
from elastos_adenine.common import Common

host = config('GRPC_SERVER_HOST')
port = config('GRPC_SERVER_PORT')
production = config('PRODUCTION', default=False, cast=bool)

network = "gmunet"
ela_to_use = 'EQeMkfRk3JzePY7zpUSg5ZSvNsWedzqWXN'
ela_eth_to_use = '0x48F01b2f2b1a546927ee99dD03dCa37ff19cB84e'


def test_node_rpc():
    # test_node_rpc
    # assert response.status==True,"Testing test_node_rpc Failed"
    pass
